/* This tool was developed at the University of Delaware and is publicly
   available for academic and non-commercial use. University of Delaware
   assumes no responsibility for consequences arising from the use of
   this tool. It is released under the GPL license. All questions and
   feedback should be directed to Jelena Mirkovic <sunshine@cis.udel.edu> */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "pcap_packet_read.h"
#include "customize.h"

#include "hash.h"
#include "pair.h"

struct hash_table* pairtable;

/* Explains the usage of the program
   input: Name of the program
   output: A warning about using the program 
   and parameters are printed to the screen */

void usage(char* prg)
{
  printf("Usage %s tcpdump-input-file tcpdump-output-file oneway-file\n", prg);
  exit(1);
}

/* Reads each line in oneway file and inserts them to pair table
   input: Name of the file 
   output: All of the pairs in the oneway file is inserted to the pair table */

void readOneWay(char* filename)
{
  FILE* oneway = fopen(filename,"r");

  if (!oneway)
    {
      printf("No one-way file %s\n", filename);
      exit(1);
    }

  /* Oneway pairs are read and inserted to the pair table */  
  while(TRUE)
    {
      struct pair_key pkey;
      int outcome, port1, port2;

      fscanf(oneway, "%u %u %u %u\n", &pkey.ip1, &port1,
	     &pkey.ip2, &port2);
      if (feof(oneway))
	break;
      insert(&pkey, pairtable, &outcome);
    }
}

/* Looks for the pair key in pair table, if it can't be found there, packet
   is written to the output file
   input: time, packet type, source IP, destination IP
   source port number, destination port number, sequence number
   acknowledgement number, length of the packet, a variable that 
   shows if syn flag is set, a variable that shows if fin flag is set,
   a variable that shows if push flag is set, a variable that shows if 
   reset flag is set, a variable that shows if ack flag is set,a variable 
   that shows if packet is fragmented, packet header and the packet
   output: connection table is updated */

void process_packet(double time, enum ptype packettype, unsigned int source, 
		    unsigned int dest, unsigned short sport, 
		    unsigned short dport, unsigned int seqnum, 
		    unsigned int acknum, int len, int syn, int fin, 
		    int push, int reset, int ack, int fragmented, 
		    const struct pcap_pkthdr *hdr, const u_char* packet)
{

  /* Pair table keys */
  struct pair_key pkey, rkey;
  int location;

  /* Initialize keys */
  pkey.ip1 = source;
  pkey.ip2 = dest;
  rkey.ip2 = source;
  rkey.ip1 = dest;

  if (find(&pkey, pairtable) == NOT_FOUND &&
      find(&rkey, pairtable) == NOT_FOUND)
    write_pcap(hdr, packet, 0);
  //else
  //printf("Found %u %u in pair table\n", source, dest);
}


int main(int argc, char** argv)
{
  if (argc != 4)
    usage(argv[0]);

  /* Hash tables is being created */
  create_hash(&pairtable,CONN_TABLE_SIZE,CONN_TABLE_SIZE,CONN_TABLE_SIZE,1,0,
	      sizeof(struct pair_key),sizeof(struct pair_data),
	      (int (*)(void *)) pair_clean,(int (*)(void *,void *))pair_compare,
	      ( unsigned long(*)(void  *,unsigned long ))pair_hash1,
	      (unsigned long(*)(unsigned long, void*,unsigned long))pair_hash2);

  /* Reads oneway pairs from the file */
  readOneWay(argv[3]);
 
  read_pcap(2, argv); 
}











