/* This tool was developed at the University of Delaware and is publicly
   available for academic and non-commercial use. University of Delaware
   assumes no responsibility for consequences arising from the use of
   this tool. It is released under the GPL license. All questions and
   feedback should be directed to Jelena Mirkovic <sunshine@cis.udel.edu> */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h> 
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>

#include "constants.h"
#include "pcap_packet_read.h"

int wfs = 0;
pcap_dumper_t* writefile[2];
pcap_t* descr;

int nonIP = 0, nonTCPUDP = 0;

unsigned int curlen = 0, packets=0;

void readPacket(u_char *args, const struct pcap_pkthdr *hdr,
		     const u_char *packet)
{
  struct ether_header *eptr;  
  struct ip* ip;
  int i;
  char source[20], dest[20];
  struct tcphdr* tcp;
  struct udphdr* udp;
  unsigned int srcIP, dstIP;
  unsigned short sport, dport;
  int len, frag;
  double dtime;

  /* lets start with the ether header... */
  eptr = (struct ether_header *) packet;
  dtime = hdr->ts.tv_sec + (double)hdr->ts.tv_usec/1000000; 
  /* Do a couple of checks to see what packet type we have..*/
  if (ntohs (eptr->ether_type) == ETHERTYPE_IP)
    {
      /* OK to proceed */
    }
  else  if (ntohs (eptr->ether_type) == ETHERTYPE_ARP)
    {
      nonIP++;
      return;
    }
  else 
    {
      /* Now a very crude fix, since it seems that some interfaces add two
	 leding 0 bytes to everything they capture */
      packet = packet + 2;
      eptr = (struct ether_header *) packet;
      if (ntohs (eptr->ether_type) == ETHERTYPE_IP)
	{
	  /* OK to proceed */
	}
      else
	{
	  /* Check for VLAN */
	  packet = packet + 2;
	  eptr = (struct ether_header *) packet;
	  if (ntohs (eptr->ether_type) == ETHERTYPE_IP)
	    {
	      /* OK to proceed */
	      //printf("VLAN\n");
	    }
	  else
	    {
	      //printf("Not ether\n");
	      nonIP++;
	      /* Give up, not an IP packet */
	      return;
	    }
	}
    }
  ip = (struct ip*)(packet + sizeof(struct ether_header));
  len = ntohs(ip->ip_len) - 4*ip->ip_hl;
  srcIP = ntohl(ip->ip_src.s_addr);
  dstIP = ntohl(ip->ip_dst.s_addr);
  frag = (ntohs(ip->ip_off) & 8192) || ((ntohs(ip->ip_off) & 8191) > 0);
  if (ip->ip_p == IPPROTO_TCP)
    {
      unsigned int seq, ack;
      tcp = (struct tcphdr*)((unsigned int)ip+4*ip->ip_hl);
      sport = ntohs(tcp->source);
      dport = ntohs(tcp->dest);
      seq = ntohl(tcp->seq);
      ack = ntohl(tcp->ack_seq);
      len = len - tcp->doff*4;
      process_packet(dtime, TCP, srcIP, dstIP, sport, dport, seq, ack, 
		     len, tcp->syn, tcp->fin, tcp->psh, tcp->rst, 
		     tcp->ack, frag, hdr, packet); 
    }
  else if (ip->ip_p == IPPROTO_UDP)
    {
      udp = (struct udphdr*)((unsigned int)ip+4*ip->ip_hl);
      sport = ntohs(udp->source);
      dport = ntohs(udp->dest);
      len = len - 8;
      process_packet(dtime, UDP, srcIP, dstIP, sport, dport, 0, 0, 
		     len, 0, 0, 0, 0, 0, frag, hdr, packet);
    }
  else if (ip->ip_p == IPPROTO_ICMP)
    {
      len = len - 8;
      process_packet(dtime, ICMP, srcIP, dstIP, 0, 0, 0, 0, 
		     len, 0, 0, 0, 0, 0, frag, hdr, packet);
    }
  else
    {
      len = len - 8;
      process_packet(dtime, ip->ip_p, srcIP, dstIP, 0, 0, 0, 0, 
		     len, 0, 0, 0, 0, 0, frag, hdr, packet);
    }
}

int read_pcap(int argc, char **argv)
{
  int i, index;
  char *dev; 
  char errbuf[PCAP_ERRBUF_SIZE];
  const u_char *packet;

  u_char *ptr; /* printing out hardware header info */

  printf("Will open %s\n", argv[1]);
  descr = pcap_open_offline(argv[1],errbuf); 

  if(descr == NULL)
    {
      printf("pcap_open_offline(): %s\n",errbuf);
      exit(1);
    }
  fflush(stdout);
  if (argc >= 2)
  open_for_writing(argv[2], 0);
  if (argc == 3)
    open_for_writing(argv[3], 1);
  pcap_loop(descr,-1,readPacket,0);
  for(index = 0; index < wfs;index++)
    {
      //pcap_dump_flush(writefile[index]);
      pcap_dump_close(writefile[index]);
    }
  return 0;

}

void open_for_writing(char* filename, int index)
{
  char errbuf[PCAP_ERRBUF_SIZE];
  writefile[index] = pcap_dump_open(descr, filename);
  wfs++;
}

int write_pcap(const struct pcap_pkthdr *hdr, u_char* packet, int index)
{
  pcap_dump((u_char*)writefile[index], hdr, packet);
  return 0;
}


